# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3. 
# See LICENSE comming with the source of 'mpuls offline'
# for details.
#
# author: Sascha L. Teichmann <sascha.teichmann@intevation.de>
#
# Converts WASKO v1 to v2 documents
#  old:
#      geschw_w_a
#      geschw_m_a
#      geschw_hh_a
#      geschw_w_b
#      geschw_m_b
#      geschw_hh_b
#      geschw_w_c
#      geschw_m_c
#      geschw_hh_c
#      geschw_w_d
#      geschw_m_d
#      geschw_hh_d
#  
#  new:
#      geschw_w
#      geschw_m
#      geschw_hh
#      geschw_alt_a
#      geschw_alt_b
#      geschw_alt_c
#      geschw_alt_d
#  
#  old -> new:
#      geschw_w     = geschw_w_a  + geschw_w_b  + geschw_w_c  + geschw_w_d
#      geschw_m     = geschw_m_a  + geschw_m_b  + geschw_m_c  + geschw_m_d
#      geschw_hh_a  = geschw_hh_a + geschw_hh_b + geschw_hh_c + geschw_hh_d
#      geschw_alt_a = geschw_w_a  + geschw_m_a
#      geschw_alt_b = geschw_w_b  + geschw_m_b
#      geschw_alt_c = geschw_w_c  + geschw_m_c
#      geschw_alt_d = geschw_w_d  + geschw_m_d

from offline.lib.xmlhelper import decode_int


def getConverterFactory():
    return WaskoV1V2

OLD_ITEMS = frozenset([
    'geschw_w_a', 'geschw_m_a', 'geschw_hh_a',
    'geschw_w_b', 'geschw_m_b', 'geschw_hh_b',
    'geschw_w_c', 'geschw_m_c', 'geschw_hh_c',
    'geschw_w_d', 'geschw_m_d', 'geschw_hh_d'])

ADDS = (
    ('geschw_w',     ('geschw_w_a',  'geschw_w_b',  'geschw_w_c',  'geschw_w_d')),
    ('geschw_m',     ('geschw_m_a',  'geschw_m_b',  'geschw_m_c',  'geschw_m_d')),
    ('geschw_hh',    ('geschw_hh_a', 'geschw_hh_b', 'geschw_hh_c', 'geschw_hh_d')),
    ('geschw_alt_a', ('geschw_w_a',  'geschw_m_a')),
    ('geschw_alt_b', ('geschw_w_b',  'geschw_m_b')),
    ('geschw_alt_c', ('geschw_w_c',  'geschw_m_c')),
    ('geschw_alt_d', ('geschw_w_d',  'geschw_m_d')))

def add(a, b):
    if a is None: return b
    if b is None: return a
    return a + b

class WaskoV1V2(object):

    def __init__(self):
        self.data        = {}
        self.nat_staat   = None
        self.kf_meth_c_1 = None
        self.kf_meth_f_1 = None

    def eat(self, name, attrs, value):

        if name == 'kf_meth_c_1':
            self.kf_meth_c_1 = value
            return True

        if name == 'kf_meth_f_1':
            self.kf_meth_f_1 = value
            return True

        if name == 'nat_staat':
            self.nat_staat = value
            return True

        if name == 'nat_staat_t': # do not import
            return True

        if name in OLD_ITEMS:
            value = decode_int(value)
            if not value is None:
                self.data[name] = value
            return True
        return False

    def finish(self, master_loader):

        if not self.kf_meth_c_1 is None \
        or not self.kf_meth_f_1 is None:
            if not self.kf_meth_f_1 is None: value = self.kf_meth_f_1
            else:                            value = self.kf_meth_c_1
            master_loader.new_data['kf_meth_f_1'] = value

        if not self.nat_staat is None:
            nat_staat = decode_int(self.nat_staat)
            if master_loader.version < 2:
                if nat_staat == 29: # deutsch
                    nat_staat = 1
                elif nat_staat != -1 or nat_staat != -3:
                    nat_staat = 2 # andere
            master_loader.new_data['nat_staat'] = nat_staat

        if self.data:
            for key, values in ADDS:
                s = reduce(add, map(lambda v: self.data.get(v), values), None)
                if not s is None: 
                    master_loader.new_data[key] = s

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
