# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3. 
# See LICENSE comming with the source of 'mpuls offline'
# for details.
#
# author: Sascha L. Teichmann <sascha.teichmann@intevation.de>
#

import os.path

from   codecs    import lookup
from   mimetypes import guess_type

IDENTITY = lambda x: x
__BASE64 = lookup('base64')[0]
BASE64   = lambda x: __BASE64(x)[0]

UTF8 = lambda x: unicode(x, 'utf-8')

class FileCache(object):

    def __init__(self, base):
        self.base  = base
        self.cache = {}

    def get_image(self, name, attributes = ""):
        ctype = guess_type(name)[0]
        return u"".join([
            u'<img src="data:', not ctype and 'application/octet-stream' or ctype, 
            u';base64,', self.get_file(name, BASE64), u'" ', attributes, u'>'])

    def get_base(self):
        return self.base

    def get_file(self, filename, encoder = IDENTITY):
        filename = os.path.normpath(os.path.join(self.base, filename))
        try:
            entry = self.cache[filename]
        except KeyError:
            f = None
            try:
                f = open(filename, "rb")
                content = f.read()
                entry = { 'content': content }
                self.cache[filename] = entry
            finally:
                if f:
                    try: f.close()
                    except: pass
        try:
            content = entry[encoder]
        except KeyError:
            content = encoder(entry['content'])
            entry[encoder] = content

        return content

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
