# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3. 
# See LICENSE comming with the source of 'mpuls offline'
# for details.
#
# author: Sascha L. Teichmann <sascha.teichmann@intevation.de>
#

import BaseHTTPServer
import posixpath
import urlparse
import urllib
import os
import mimetypes
import shutil
import threading

DEFAULT_PORT = 8234

def run(base, port = DEFAULT_PORT):
    server_addr   = ('', port)
    httpd = FileHTTPServer(
        server_addr,
        FileHTTPRequestHandler,
        base)
    thread = ServerThread(httpd)
    thread.start()

class ServerThread(threading.Thread):

    def __init__(self, httpd):
        threading.Thread.__init__(self)
        self.httpd = httpd
        self.setDaemon(True)

    def run(self):
        self.httpd.serve_forever()

class FileHTTPServer(BaseHTTPServer.HTTPServer):

    def __init__(self, server_address, handler_class, base):
        BaseHTTPServer.HTTPServer.__init__(self, server_address, handler_class)
        self.base = base

class FileHTTPRequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):

    def do_GET(self):
        """Serve a GET request."""
        f = self.send_head()
        if f:
            shutil.copyfileobj(f, self.wfile)
            f.close()

    def do_HEAD(self):
        """Serve a HEAD request."""
        f = self.send_head()
        if f:
            f.close()

    def send_head(self):
        host = self.client_address[0]
        if not host in ("127.0.0.1", "localhost", "::1"):
            return None

        path = self.translate_path(self.path)
        f = None
        try:
            f = open(path, 'rb')
        except IOError:
            self.send_error(404, "File not found")
            return None
        ctype = mimetypes.guess_type(path)[0]
        if not ctype:
            ctype = "application/octet-stream"
        self.send_response(200)
        self.send_header("Content-type", ctype)
        fs = os.fstat(f.fileno())
        self.send_header("Content-Length", str(fs[6]))
        self.send_header("Last-Modified", self.date_time_string(fs.st_mtime))
        self.end_headers()
        return f

    def translate_path(self, path):
        path = urlparse.urlparse(path)[2]
        path = posixpath.normpath(urllib.unquote(path))
        print path
        words = path.split('/')
        words = filter(None, words)
        path = self.server.base
        for word in words:
            drive, word = os.path.splitdrive(word)
            head, word = os.path.split(word)
            if word in (os.curdir, os.pardir): continue
            path = os.path.join(path, word)
        return path

if __name__ == '__main__':
    run(os.getcwd())
    raw_input("Press enter")

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
