# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3. 
# See LICENSE comming with the source of 'mpuls offline'
# for details.
#
# author: Sascha L. Teichmann <sascha.teichmann@intevation.de>
#

from optparse import OptionParser

from PyQt4.QtGui import QApplication

from config import Config

from io.document import openDocument

from model.memory      import MemoryDatastore
from model.defaults    import Defaults
from model.annotations import AnnotationsProvider
from model.logbook     import LogbookMeta

from gui import MainWindow

from urllib import pathname2url

import sys
import os
import imp
import datetime

from lib.httpserver import run as run_httpd
from lib.filecache  import FileCache

def main_is_frozen():
    return (
        hasattr(sys, "frozen")    or # new py2exe
        hasattr(sys, "importers") or # old py2exe
        imp.is_frozen("__main__"))   # tools/freeze

class Main(object):

    def __init__(self):
        parser = OptionParser()
        parser.add_option(
            "-c", "--config",
            help    = "Path to contiguration file",
            dest    = "config",
            default = "offline.cfg")

        (options, args) = parser.parse_args()

        base = main_is_frozen() \
            and os.path.dirname(sys.argv[0]) \
            or  os.path.normpath(os.path.dirname(__file__))

        self.config = Config(base, options.config)

        formed_path = self.config.get_formed_path()
        # load FormEd
        self.formed = openDocument(formed_path)

        converter_mod = self.config.get_converter_module()
        if converter_mod:
            mod = __import__(converter_mod, globals(), locals(), ['getConverterFactory'])
            self.converter_factory = mod.getConverterFactory()
        else:
            self.converter_factory = None

        static_base    = os.path.join(base, "static")
        resources_base = os.path.join(base, "resources")

        self.file_cache  = FileCache(static_base)
        self.base_url    = pathname2url(static_base + "/")
        self.defaults    = Defaults(resources_base, "defaults.xml")
        self.annotations = AnnotationsProvider(
            os.path.join(resources_base, "annotations.xhtml"))

        self.logbook_meta = LogbookMeta(
            os.path.join(resources_base, "logbook-meta.xml"))

        self.document = args \
            and MemoryDatastore(
                self.formed,
                self.defaults,
                self.logbook_meta,
                self.converter_factory,
                args[0]) \
            or None

        if self.document:
            swapfile = os.path.join(
                self.config.get_homedir(),
                self.config.get_swapfile())
            self.document.setSwapfile(swapfile)

def main():
    app = QApplication(sys.argv)
    main = Main()
    main_window = MainWindow(main)
    main_window.show()
    sys.exit(app.exec_())

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
