# -*- coding: utf-8 -*-
#
# (c) 2008 by Intevation GmbH
# This is Free software under the GPLv3. 
# See LICENSE comming with the source of 'mpuls offline'
# for details.
#
# author: Sascha L. Teichmann <sascha.teichmann@intevation.de>
#

from xml.dom.minidom import parse

import re
import os

REQUIRED   = ("required", )
HELP       = ("help", )
EVALUATION = ("evaluation", )

ANNOTATION_DIV_ID = re.compile("([a-zA-Z0-9_\-]+):([a-zA-Z0-9_\-]+):([a-zA-Z0-9_\-,]+)")

class Annotation:

    def __init__(self, kind, msg):
        self.kind = kind
        self.msg  = msg

    def __str__(self):
        return repr(self.kind) + ": " + repr(self.msg)

    def __repr__(self):
        return self.__str__()

def build_annotation(annotations_file):

    annotations = {}
    dom = None
    try:
        dom = parse(annotations_file)
        for div in dom.getElementsByTagName("div"):
            id = div.getAttribute("id")
            if not id: continue
            m = ANNOTATION_DIV_ID.match(id)
            if not m: continue
            id, kind = m.group(1), m.group(2)
            items = [item for item in m.group(3).split(",") if item]
            if not items: continue
            div.setAttribute("id", id)

            msg = unicode(div.toxml())

            for item in items:
                annons = annotations.setdefault(item, [])
                annons.append(Annotation(kind, msg))
    finally:
        if dom: dom.unlink()

    return annotations


class AnnotationsProvider:

    def __init__(self, annotations_file):
        if os.path.isfile(annotations_file):
            self.annotations = build_annotation(annotations_file)
        else:
            self.annotations = {}

    def hasHelp(self, id):
        return self.hasAnnotations(id, HELP)

    def getHelp(self, id):
        return self.getAnnotations(id, HELP)
        
    def hasEvaluation(self, id):
        return self.hasAnnotations(id, EVALUATION)

    def getEvaluation(self, id):
        return self.getAnnotations(id, EVALUATION)
        
    def hasAnnotations(self, id, kinds = REQUIRED):
        annotations = self.annotations.get(id)
        if not annotations: return False
        for annotation in annotations:
            if annotation.kind in kinds:
                return True
        return False

    def getAnnotations(self, id, kinds = REQUIRED):
        try:
            x = [a.msg for a in self.annotations[id] if a.kind in kinds]
        except KeyError:
            x = []
        return u''.join(x)

# vim:set ts=4 sw=4 si et sta sts=4 fenc=utf8 :
